@@ -1,4 +1,4 @@
-__version__ = '0.15'
+__version__ = '0.16'
 
 from usertools import monitor, timing
 
@@ -113,6 +113,8 @@ nprod = mp.nprod
 difference = mp.difference
 diff = mp.diff
 diffs = mp.diffs
+diffs_prod = mp.diffs_prod
+diffs_exp = mp.diffs_exp
 diffun = mp.diffun
 differint = mp.differint
 taylor = mp.taylor
@@ -122,6 +124,7 @@ polyroots = mp.polyroots
 fourier = mp.fourier
 fourierval = mp.fourierval
 sumem = mp.sumem
+sumap = mp.sumap
 chebyfit = mp.chebyfit
 limit = mp.limit
 
@@ -322,8 +325,19 @@ fresnels = mp.fresnels
 fresnelc = mp.fresnelc
 airyai = mp.airyai
 airybi = mp.airybi
-ellipe = mp.ellipe
+airyaizero = mp.airyaizero
+airybizero = mp.airybizero
+scorergi = mp.scorergi
+scorerhi = mp.scorerhi
 ellipk = mp.ellipk
+ellipe = mp.ellipe
+ellipf = mp.ellipf
+ellippi = mp.ellippi
+elliprc = mp.elliprc
+elliprj = mp.elliprj
+elliprf = mp.elliprf
+elliprd = mp.elliprd
+elliprg = mp.elliprg
 agm = mp.agm
 jacobi = mp.jacobi
 chebyt = mp.chebyt
@@ -332,6 +346,10 @@ legendre = mp.legendre
 legenp = mp.legenp
 legenq = mp.legenq
 hermite = mp.hermite
+pcfd = mp.pcfd
+pcfu = mp.pcfu
+pcfv = mp.pcfv
+pcfw = mp.pcfw
 gegenbauer = mp.gegenbauer
 laguerre = mp.laguerre
 spherharm = mp.spherharm
@@ -341,10 +359,16 @@ j1 = mp.j1
 besseli = mp.besseli
 bessely = mp.bessely
 besselk = mp.besselk
+besseljzero = mp.besseljzero
+besselyzero = mp.besselyzero
 hankel1 = mp.hankel1
 hankel2 = mp.hankel2
 struveh = mp.struveh
 struvel = mp.struvel
+angerj = mp.angerj
+webere = mp.webere
+lommels1 = mp.lommels1
+lommels2 = mp.lommels2
 whitm = mp.whitm
 whitw = mp.whitw
 ber = mp.ber
