@@ -2,6 +2,10 @@ import functions
 # Hack to update methods
 import factorials
 import hypergeometric
+import expintegrals
+import bessel
+import orthogonal
+import theta
 import elliptic
 import zeta
 import rszeta
